local USE_SFX = Resources.load_audio("use_shuffle.ogg")
local PREVIEW_TEXTURE = Resources.load_texture("preview.png")
local BUTTON_TEXTURE = Resources.load_texture("button.png")

function augment_init(augment)
  local player = augment:owner()

  local button = augment:create_card_button(2)
  button:use_default_audio(false)
  button:set_texture(BUTTON_TEXTURE)
  button:set_preview_texture(PREVIEW_TEXTURE)

  local animation = button:animation()
  animation:load("button.animation")

  local used = false

  augment._card_list = {}

  local open_component = player:create_component(Lifetime.CardSelectOpen)
  open_component.on_update_func = function()
    if used == false then animation:set_state("DEFAULT") else animation:set_state("DISABLED") end
  end

  button.use_func = function(self)
    if used then
      return false
    end

    local staged_items = player:staged_items()

    for _, item in ipairs(staged_items) do
      if item.category == "deck_card" then
        local card = player:deck_card(item.index)
        local properties = CardProperties.from_package(card.package_id, card.code)

        if properties.time_freeze ~= true then
          table.insert(augment._card_list, { id = card.package_id, props = properties, deck_card = item })
          player:stage_deck_discard(item.index)
          -- It's so funny that I can do this lol
          -- This instant closes the custom menu...! I love it.
          -- player:confirm_staged_items()
        end
      end
    end

    local action_properties = CardProperties.from_package("dev.GladeWoodsgrove.CustomAdvance.Chip")

    action_properties.time_freeze = true
    action_properties.short_name = "Fused Attack"
    action_properties.can_boost = false
    action_properties.can_charge = false
    action_properties.skip_time_freeze_intro = true

    local storage_aux_prop = AuxProp.new()
    storage_aux_prop:once()
    storage_aux_prop:require_card_tag("GLADE_CUSTOM_FUSION_CARD")
    storage_aux_prop:require_card_element(Element.None)

    storage_aux_prop._storage_list = {}

    for i = 1, #augment._card_list, 1 do
      table.insert(storage_aux_prop._storage_list, augment._card_list[i])
    end

    storage_aux_prop:intercept_action(function()
      local action = Action.new(player)

      action.on_execute_func = function()
        local list = storage_aux_prop._storage_list

        for i = 1, #list, 1 do
          if (CardProperties.from_package(list[i].id)) ~= nil then
            local queued_action = Action.from_card(player, list[i].props)
            if queued_action ~= nil then player:queue_action(queued_action) end
          end
        end
      end

      return action
    end)


    player:stage_card(action_properties, function()
      local list = augment._card_list
      for i = 1, #list, 1 do
        player:stage_deck_card(list[i].deck_card.index)
      end
      used = false
      animation:set_state("DEFAULT")
      augment._card_list = {}
    end)

    local close_component = player:create_component(Lifetime.Local)
    close_component.on_update_func = function(self)
      for index, field_card in ipairs(player:field_cards()) do
        if field_card.package_id == "dev.GladeWoodsgrove.CustomAdvance.Chip" then
          player:add_aux_prop(storage_aux_prop);
          break;
        end
      end
      self:eject()
    end

    used = true

    animation:set_state("USED")

    player:play_audio(USE_SFX)

    return true
  end
end
